import { __extends } from "tslib";
import { CronExpression, Expression, StringLiteral } from "@/syntax/cron";
import { rangeExpr, stepExpr } from "@/syntax/common";
var abstractVisitor = function (base) {
    var Visitor = /** @class */ (function (_super) {
        __extends(Visitor, _super);
        function Visitor() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _this = _super.call(this, args) || this;
            _this.validateVisitor();
            return _this;
        }
        Visitor.prototype.cronExpression = function (ctx) {
            var visitedContext = new CronExpression();
            visitedContext.minute = this.visit(ctx.minutes);
            visitedContext.hour = this.visit(ctx.hours);
            visitedContext.dom = this.visit(ctx.dom);
            visitedContext.month = this.visit(ctx.month);
            visitedContext.dow = this.visit(ctx.dow);
            return visitedContext;
        };
        Visitor.prototype.expression = function (ctx) {
            var _this = this;
            var exprs = ctx.exprNotUnion.map(function (e) { return _this.visit(e); });
            return new Expression(exprs);
        };
        Visitor.prototype.exprNotUnion = function (ctx) {
            var lhs = new StringLiteral(ctx.lhs[0].image);
            return this.visit(ctx.atomicExpr, lhs);
        };
        Visitor.prototype.atomicExpr = function (ctx, lhs) {
            var expr = lhs;
            if (ctx.range) {
                var rhs = this.visit(ctx.range);
                var leftValue = Number(lhs.value());
                var rightValue = Number(rhs.value());
                if (!isNaN(leftValue) && !isNaN(rightValue)) {
                    if (leftValue > rightValue) {
                        throw new Error("Left-hand side range value must be smaller than right-hand side");
                    }
                }
                expr = rangeExpr(lhs, rhs);
            }
            if (ctx.interval) {
                expr = stepExpr(expr, this.visit(ctx.interval));
            }
            return expr;
        };
        Visitor.prototype.interval = function (ctx) {
            return new StringLiteral(ctx.rhs[0].image);
        };
        Visitor.prototype.range = function (ctx) {
            return new StringLiteral(ctx.rhs[0].image);
        };
        return Visitor;
    }(base));
    return Visitor;
};
export default abstractVisitor;
