import { __extends } from "tslib";
import { QuartzParser } from "@/parser";
import { StringLiteral } from "@/syntax/cron";
import { DayOfWeekExpr, QuartzCronExpression } from "@/syntax/quartz";
import abstractVisitor from "./AbstractVisitorConstructor";
var QuartzVisitorConstructor = abstractVisitor(new QuartzParser().getBaseCstVisitorConstructor());
var QuartzVisitor = /** @class */ (function (_super) {
    __extends(QuartzVisitor, _super);
    function QuartzVisitor() {
        return _super.call(this, QuartzVisitorConstructor) || this;
    }
    QuartzVisitor.prototype.cronExpression = function (ctx) {
        var visitedContext = new QuartzCronExpression();
        if (ctx.second) {
            visitedContext.second = this.visit(ctx.second);
        }
        visitedContext.minute = this.visit(ctx.minutes);
        visitedContext.hour = this.visit(ctx.hours);
        visitedContext.dom = this.visit(ctx.dom);
        visitedContext.month = this.visit(ctx.month);
        visitedContext.dow = this.visit(ctx.dow);
        if (ctx.year) {
            visitedContext.year = this.visit(ctx.year);
        }
        return visitedContext;
    };
    QuartzVisitor.prototype.exprNotUnion = function (ctx) {
        var lhs = new StringLiteral(ctx.lhs[0].image);
        if (ctx.dow) {
            return this.visit(ctx.dow, lhs);
        }
        return this.visit(ctx.atomicExpr, lhs);
    };
    QuartzVisitor.prototype.dow = function (ctx, lhs) {
        var occurrence = ctx.occurence[0].image === "L" ? 5 : parseInt(ctx.occurence[0].image, 10);
        return new DayOfWeekExpr(lhs, occurrence);
    };
    return QuartzVisitor;
}(QuartzVisitorConstructor));
export default QuartzVisitor;
